@@ -1,12 +1,11 @@
 # Standard Libraries
-import os
 import random
 import numpy as np
 from copy import deepcopy
 
 # PyXtal imports #avoid *
 from pyxtal.symmetry import Group, choose_wyckoff
-from pyxtal.wyckoff_site import atom_site, check_atom_sites, WP_merge
+from pyxtal.wyckoff_site import atom_site, WP_merge
 from pyxtal.msg import printx
 from pyxtal.tolerance import Tol_matrix
 from pyxtal.lattice import Lattice, cellsize
@@ -37,27 +36,45 @@ class random_crystal:
 
     def __init__(
         self,
-        group,
-        species,
-        numIons,
+        group=None,
+        species=None,
+        numIons=None,
         factor=1.1,
         lattice=None,
         sites = None,
+        conventional = True,
         tm=Tol_matrix(prototype="atomic"),
     ):
 
         self.dim = 3 #periodic dimensions of the crystal
+        self.PBC = [1, 1, 1] #The periodic boundary axes of the crystal
         if type(group) != Group:
             group = Group(group, self.dim)
-        self.sg = group.number #The international spacegroup number 
-        self.PBC = [1, 1, 1]
-        """The periodic boundary axes of the crystal"""
-        self.init_common(species, numIons, factor, group, lattice, sites, tm)
+        self.init_common(species, numIons, factor, group, lattice, sites, conventional, tm)
 
-    def init_common(self, species, numIons, factor, group, lattice, sites, tm):
+    def __str__(self):
+        if self.valid:
+            s = "------Crystal from {:s}------".format(self.source)
+            s += "\nComposition: {}".format(self.formula)
+            s += "\nDimension: {}".format(self.dim)
+            s += "\nGroup: {} ({})".format(self.group.symbol, self.group.number)
+            s += "\n{}".format(self.lattice)
+            s += "\nWyckoff sites:"
+            for wyc in self.atom_sites:
+                s += "\n\t{}".format(wyc)
+        else:
+            s += "\nStructure not available."
+        return s
+
+    def __repr__(self):
+        return str(self)
+
+
+    def init_common(self, species, numIons, factor, group, lattice, sites, conventional, tm):
         """
         Common init functionality for 0D-3D cases of random_crystal.
         """
+        self.source = 'Random'
         self.valid = False
         # Check that numIons are integers greater than 0
         for num in numIons:
@@ -89,15 +106,18 @@ class random_crystal:
         self.numattempts = 0
         numIons = np.array(numIons)
         self.factor = factor
-        self.numIons0 = numIons
-        self.numIons = self.numIons0 * cellsize(self.group)
+        if not conventional:
+            mul = cellsize(self.group)
+        else:
+            mul = 1
+        self.numIons = numIons * mul
+
         formula = ""
         for i, s in zip(self.numIons, species):
             formula += "{:s}{:d}".format(s, int(i))
         self.formula = formula
 
         self.species = species
-        self.Msgs()
 
         # Use the provided lattice
         if lattice is not None:
@@ -282,15 +302,13 @@ class random_crystal:
                             if n[p] == 0 and p == p2:
                                 p2 = p + 1
                             break
-        # All species passed: return True
-        if has_freedom is True:
+        if has_freedom:
+            # All species passed: return True
             return True
-        # All species passed, but no degrees of freedom: return 0
-        elif has_freedom is False:
+        else:
+            # All species passed, but no degrees of freedom: return 0
             return 0
     
-
-
     def check_consistency(self, site, numIon):
         num = 0
         for s in site:
@@ -303,48 +321,6 @@ class random_crystal:
             msg += "\nfrom Wyckoff list: {:d}".format(num)
             raise ValueError(msg)
 
-    def check_short_distances(self, r=0.7, exclude_H = True):
-        """
-        A function to check short distance pairs
-        Mainly used for debug, powered by pymatgen
-
-        Args:
-            r: the given cutoff distances
-            exclude_H: whether or not exclude the H atoms
-
-        Returns:
-            list of pairs within the cutoff
-        """
-        if dim > 0:
-            pairs = []
-            pmg_struc = self.to_pymatgen()
-            if exclude_H:
-                pmg_struc.remove_species('H')
-            res = pmg_struc.get_all_neighbors(r)
-            for i, neighs in enumerate(res):
-                for n in neighs:
-                    pairs.append([pmg_struc.sites[i].specie, n.specie, n.nn_distance])
-        else:
-            raise NotImplementedError("Does not support cluster for now")
-        return pairs
-
-    def Msgs(self):
-        """
-        Define a set of error and warning message if generation fails.
-
-        Returns:
-            nothing
-        """
-        self.Msg1 = (
-            "Error: the stoichiometry is incompatible with the wyckoff sites choice"
-        )
-        self.Msg2 = "Error: failed in the cycle of generating structures"
-        self.Msg3 = "Warning: failed in the cycle of adding species"
-        self.Msg4 = "Warning: failed in the cycle of choosing wyckoff sites"
-        self.Msg5 = "Finishing: added the specie"
-        self.Msg6 = "Finishing: added the whole structure"
-        self.Msg7 = "Error: invalid paramaters for initialization"
-
     def estimate_volume(self):
         """
         Estimates the volume of a unit cell based on the number and types of ions.
@@ -362,61 +338,6 @@ class random_crystal:
             volume += numIon * 4 / 3 * np.pi * r ** 3
         return self.factor * volume
 
-    def to_file(self, filename=None, fmt=None, permission='w'):
-        """
-        Creates a file with the given filename and file type to store the structure.
-        By default, creates cif files for crystals and xyz files for clusters.
-        For other formats, Pymatgen is used
-
-        Args:
-            filename: the file path
-            fmt: the file type (`cif`, `xyz`, etc.)
-            permission: `w` or `a+`
-
-        Returns:
-            Nothing. Creates a file at the specified path
-        """
-        if self.valid:
-            if fmt is None:
-                if self.dim == 0:
-                    fmt = 'xyz'
-                else:
-                    fmt = 'cif'
-
-            if fmt == "cif":
-                from pyxtal.io import write_cif
-                return write_cif(self, filename, "from_pyxtal", permission)
-            else:
-                pmg_struc = self.to_pymatgen()
-                return pmg_struc.to(fmt=fmt, filename=filename)
-        else:
-            printx("Cannot create file: structure did not generate.", priority=1)
-
-    def __str__(self):
-        s = "------Random Crystal------"
-        s += "\nComposition: {}".format(self.formula)
-        s += "\nDimension: {}".format(self.dim)
-        s += "\nGroup: {} ({})".format(self.group.symbol, self.group.number)
-        s += "\nVolume factor: {}".format(self.factor)
-        s += "\n{}".format(self.lattice)
-        if self.valid:
-            s += "\nWyckoff sites:"
-            for wyc in self.atom_sites:
-                s += "\n\t{}".format(wyc)
-        else:
-            s += "\nStructure not generated."
-        return s
-
-    def __repr__(self):
-        return str(self)
-
-    def show(self, **kwargs):
-        """
-        display the crystal structure
-        """
-        from pyxtal.viz import display_atomic
-        return display_atomic(self, **kwargs)
-
     def generate_crystal(self):
         """
         The main code to generate a random atomic crystal. If successful,
@@ -429,7 +350,8 @@ class random_crystal:
         self.numattempts = 1
         degrees = self.check_compatible(self.group, self.numIons)
         if degrees is False:
-            printx(self.Msg1, priority=1)
+            msg = "Error: the stoichiometry is incompatible with wyckoff choice"
+            printx(msg, priority=1)
             self.valid = False
             return
 
@@ -491,73 +413,6 @@ class random_crystal:
 
         return
 
-    def copy(self):
-        """
-        simply copy the structure
-        """
-        return deepcopy(self)
-
-    def _get_coords_and_species(self, absolute=False):
-        """
-        extract the coordinates and species information 
-
-        Args:
-            abosulte: if True, return the cartesian coords otherwise fractional
-
-        Returns:
-            total_coords (N*3 numpy array) and the list of species
-        """
-        species = []
-        total_coords = None
-        for site in self.atom_sites:
-            species.extend([site.specie]*site.multiplicity)
-            if total_coords is None:
-                total_coords = site.coords
-            else:
-                total_coords = np.append(total_coords, site.coords, axis=0)
-
-        if absolute:
-            return total_coords.dot(self.lattice.matrix), species
-        else:
-            return total_coords, species
-
-    def to_ase(self):
-        """
-        export to ase Atoms object
-        """
-        from ase import Atoms
-        if self.valid:
-            if self.dim > 0:
-                coords, species = self._get_coords_and_species()
-                # Add space above and below a 2D or 1D crystals
-                latt, coords = self.lattice.add_vacuum(coords, PBC=self.PBC)
-                return Atoms(species, scaled_positions=coords, cell=latt)
-            else:
-                coords, species = self._get_coords_and_species(True)
-                return Atoms(species, positions=coords)
-        else:
-            printx("No valid structure can be converted to ase.", priority=1)
-
-    def to_pymatgen(self):
-        """
-        export to Pymatgen structure object
-        """
-        from pymatgen.core.structure import Structure, Molecule
-
-        if self.valid:
-            if self.dim > 0:
-                coords, species = self._get_coords_and_species()
-                # Add space above and below a 2D or 1D crystals
-                latt, coords = self.lattice.add_vacuum(coords, PBC=self.PBC)
-                return Structure(latt, species, coords)
-            else:
-                # Clusters are handled as large molecules
-                coords, species = self._get_coords_and_species(True)
-                return Molecule(species, coords)
-        else:
-            printx("No valid structure can be converted to pymatgen.", priority=1)
-
-
     def _generate_coords(self, cell_matrix):
         """
         generate coordinates for random crystal
@@ -624,13 +479,17 @@ class random_crystal:
                 # Generate a list of coords from ops
                 mult = wp.multiplicity # remember the original multiplicity
                 pt = self.lattice.generate_point()
-
                 # Merge coordinates if the atoms are close
                 pt, wp, _ = WP_merge(pt, cell_matrix, wp, tol)
-                
+                # For pure planar structure
+                if self.dim == 2 and self.thickness is not None and self.thickness < 0.1:
+                    pt[-1] = 0.5
+
+               
                 # If site the pre-assigned, do not accept merge
                 if wp is not False:
                     if site is not None and mult != wp.multiplicity:
+                        cycle += 1
                         continue 
                     # Use a Wyckoff_site object for the current site
                     new_site = atom_site(wp, pt, specie)
@@ -638,7 +497,7 @@ class random_crystal:
                     # Check current WP against existing WP's
                     passed_wp_check = True
                     for ws in wyckoff_sites_tmp + wyks:
-                        if not check_atom_sites(new_site, ws, cell_matrix, tol_matrix):
+                        if not new_site.check_with_ws2(ws, cell_matrix, tol_matrix):
                             passed_wp_check = False
 
                     if passed_wp_check:
@@ -690,6 +549,7 @@ class random_crystal_2D(random_crystal):
         thickness=None,
         lattice=None,
         sites = None,
+        conventional = True,
         tm=Tol_matrix(prototype="atomic"),
     ):
         self.dim = 2
@@ -699,7 +559,7 @@ class random_crystal_2D(random_crystal):
             group = Group(group, self.dim)
         number = group.number  # The layer group number of the crystal
         self.thickness = thickness  # in Angstroms, in the 3rd dimenion of unit cell
-        self.init_common(species, numIons, factor, number, lattice, sites, tm)
+        self.init_common(species, numIons, factor, number, lattice, sites, conventional, tm)
 
 
 class random_crystal_1D(random_crystal):
@@ -733,13 +593,13 @@ class random_crystal_1D(random_crystal):
         area=None,
         lattice=None,
         sites = None,
+        conventional = True,
         tm=Tol_matrix(prototype="atomic"),
     ):
         self.dim = 1
         self.PBC = [0, 0, 1]
-        self.sg = None
         self.area = area  # the effective cross-sectional area, in A^2, of the unit cell.
-        self.init_common(species, numIons, factor, group, lattice, sites, tm)
+        self.init_common(species, numIons, factor, group, lattice, sites, conventional, tm)
 
 
 class random_cluster(random_crystal):
@@ -775,9 +635,6 @@ class random_cluster(random_crystal):
         sites = None,
         tm=Tol_matrix(prototype="atomic", factor=0.7),
     ):
-        # NOTE tol_m unused?
-        tol_m = 0.1
         self.dim = 0
         self.PBC = [0, 0, 0]
-        self.sg = None
-        self.init_common(species, numIons, factor, group, lattice, sites, tm)
+        self.init_common(species, numIons, factor, group, lattice, sites, False, tm)
