@@ -129,6 +129,23 @@ class Lattice:
             self.stress_indices = []
         # Set values for the matrix
         self.reset_matrix()
+        self._get_dof()
+
+    def _get_dof(self):
+        """
+        get the number of degree of freedom
+        """
+        if self.ltype in ["triclinic", "Triclinic"]:
+            self.dof = 6
+        elif self.ltype in ["monoclinic", "Monoclinic"]:
+            self.dof = 4
+        elif self.ltype in ['orthorhombic', 'Orthorhombic']:
+            self.dof = 3
+        elif self.ltype in ['tetragonal', 'Tetragonal', 
+            'hexagonal', 'trigonal', 'Hexagonal', 'Trigonal']:        
+            self.dof = 2
+        else:
+            self.dof = 1
 
     def copy(self):
         """
@@ -188,32 +205,46 @@ class Lattice:
             cell = np.dot(tran, self.matrix)
             if id > 0:
                 opt = True
-            return Lattice.from_matrix(cell, ltype=self.ltype), tran, opt
+            #print(cell); import sys; sys.exit()
+            return Lattice.from_matrix(cell, ltype=self.ltype, reset=False), tran, opt
         else:
             return self, np.eye(3), opt
 
-    def mutate(self, degree=0.20):
+
+    def mutate(self, degree=0.20, frozen=False):
         """
         mutate the lattice object
         """
         rand = 1 + degree*(np.random.sample(6)-0.5)
-        a, b, c, alpha, beta, gamma = self.get_para()
-        a *= rand[0]
-        b *= rand[1]
-        c *= rand[2]
-        alpha = np.degrees(alpha*rand[3])
-        beta = np.degrees(beta*rand[4])
-        gamma = np.degrees(gamma*rand[5])
+        a0, b0, c0, alpha0, beta0, gamma0 = self.get_para()
+        a = a0*rand[0]
+        b = b0*rand[1]
+        c = c0*rand[2]
+        alpha = np.degrees(alpha0*rand[3])
+        beta = np.degrees(beta0*rand[4])
+        gamma = np.degrees(gamma0*rand[5])
         ltype = self.ltype
 
         if self.ltype in ['cubic', 'Cubic']:
-            lat = Lattice.from_para(a, a, a, 90, 90, 90, ltype=ltype)
+            if frozen:
+                lat = Lattice.from_para(a0, a0, a0, 90, 90, 90, ltype=ltype)
+            else:
+                lat = Lattice.from_para(a, a, a, 90, 90, 90, ltype=ltype)
         elif ltype in ['hexagonal', 'trigonal', 'Hexagonal', 'Trigonal']:
-            lat = Lattice.from_para(a, a, a*np.sqrt(3)/2, 90, 90, 120, ltype=ltype)
-        elif ltype in ['rhombohedral', 'Rhombohedral']:
-            lat = Lattice.from_para(a, a, a, alpha, alpha, alpha, ltype=ltype)
+            if frozen:
+                lat = Lattice.from_para(a0, a0, c, 90, 90, 120, ltype=ltype)
+            else:
+                lat = Lattice.from_para(a, a, c, 90, 90, 120, ltype=ltype)
+        #elif ltype in ['rhombohedral', 'Rhombohedral']:
+        #    if forzen:
+        #        lat = Lattice.from_para(a0, a0, a0, alpha0, alpha0, alpha0, ltype=ltype)
+        #    else:
+        #        lat = Lattice.from_para(a, a, a, alpha, alpha, alpha, ltype=ltype)
         elif ltype in ['tetragonal', 'Tetragonal']:
-            lat = Lattice.from_para(a, a, c, 90, 90, 90, ltype=ltype)
+            if frozen:
+                lat = Lattice.from_para(a0, a0, c, 90, 90, 90, ltype=ltype)
+            else:
+                lat = Lattice.from_para(a, a, c, 90, 90, 90, ltype=ltype)
         elif ltype in ['orthorhombic', 'Orthorhombic']:
             lat = Lattice.from_para(a, b, c, 90, 90, 90, ltype=ltype)
         elif ltype in ['monoclinic', 'Monoclinic']:
@@ -274,7 +305,7 @@ class Lattice:
                 self.inv_matrix = np.linalg.inv(m)
             else:
                 printx("Error: matrix must be a 3x3 numpy array or list", priority=1)
-        elif matrix is None:
+        else:
             self.reset_matrix()
         para = matrix2para(self.matrix)
         self.a, self.b, self.c, self.alpha, self.beta, self.gamma = para
@@ -305,24 +336,41 @@ class Lattice:
                     self.beta = beta
                     self.gamma = gamma
                     break
+        else:
+            # a small utility to convert the cell shape
+            para = matrix2para(self.matrix)
+            self.matrix = para2matrix(para)
 
     def set_volume(self, volume):
         if self.allow_volume_reset is True:
             self.volume = volume
 
-
+    def transform(self, tran):
+        """
+        Transform the lattice, assuming the symmetry does not change        
+        mostly designed for alternative wyckoff set
+        Args: 
+            tran: 3*3 matrix
+        """
+        # only applied to triclinic/monoclinic/orthorhombic
+        matrix = tran.dot(self.matrix)
+        (a,b,c,alpha,beta,gamma) = matrix2para(matrix)
+        alpha, beta, gamma = alpha*deg, beta*deg, gamma*deg
+        return self.from_para(a, b, c, alpha, beta, gamma, self.ltype)
+ 
     def swap_axis(self, random=False, ids=None):
         """
         For the lattice
         """
         # only applied to triclinic/monoclinic/orthorhombic
         if self.ltype in ["triclinic", "Triclinic", "orthorhombic", "Orthorhombic"]:
-            allowed_ids = [[0,1,2],[1,0,2],[0,2,1],[2,1,0]]
+            allowed_ids = [[0,1,2],[1,0,2],[0,2,1],[2,1,0],[1,2,0],[2,0,1]]
         elif self.ltype == "monoclinic":
             if abs(self.beta-90*rad) > 1e-3:
                 allowed_ids = [[0,1,2],[2,1,0]]
             else:
-                allowed_ids = [[0,1,2],[1,0,2],[0,2,1],[2,1,0]]
+                allowed_ids = [[0,1,2],[1,0,2],[0,2,1],
+                               [2,1,0],[1,2,0],[2,0,1]]
         else:
             allowed_ids = [[0,1,2]]
 
@@ -344,6 +392,10 @@ class Lattice:
             return self.from_para(c, b, a, gamma, beta, alpha, self.ltype)
         elif ids == [0,2,1]: #b-c
             return self.from_para(a, c, b, alpha, gamma, beta, self.ltype)
+        elif ids == [2,0,1]:
+            return self.from_para(c, a, b, gamma, alpha, beta, self.ltype)
+        elif ids == [1,2,0]:
+            return self.from_para(b, c, a, beta, gamma, alpha, self.ltype)
         else:
             return self
     
@@ -381,7 +433,7 @@ class Lattice:
         else:
             return self
     
-    def add_vacuum(self, coor, vacuum=10, PBC=[0, 0, 0]):
+    def add_vacuum(self, coor, frac=True, vacuum=15, PBC=[0, 0, 0]):
         """
         Adds space above and below a 2D or 1D crystal. 
     
@@ -396,12 +448,20 @@ class Lattice:
             The transformed lattice and coordinates after the vacuum is added
         """
         matrix = self.matrix
-        absolute_coords = np.dot(coor, matrix)
+        if frac:
+            absolute_coords = np.dot(coor, matrix)
+        else:
+            absolute_coords = coor
+
         for i, a in enumerate(PBC):
             if not a:
                 ratio = 1 + vacuum/np.linalg.norm(matrix[i])
                 matrix[i] *= ratio
-        coor = np.dot(absolute_coords, np.linalg.inv(matrix))
+                absolute_coords[:, i] += vacuum/2
+        if frac:
+            coor = np.dot(absolute_coords, np.linalg.inv(matrix))
+        else:
+            coor = absolute_coords 
         return matrix, coor
 
 
@@ -508,7 +568,7 @@ class Lattice:
         return l
 
     @classmethod
-    def from_matrix(self, matrix, ltype="triclinic", PBC=[1, 1, 1], **kwargs):
+    def from_matrix(self, matrix, reset=True, ltype="triclinic", PBC=[1, 1, 1], **kwargs):
         """
         Creates a Lattice object from a 3x3 cell matrix. Additional keyword arguments
         are available. Unless specified by the keyword random=True, does not create a
@@ -550,8 +610,27 @@ class Lattice:
             printx("Error: Lattice matrix must be 3x3", priority=1)
             return
         [a, b, c, alpha, beta, gamma] = matrix2para(m)
-        volume = np.linalg.det(m)
+        if reset:
+            if ltype in ['cubic', 'Cubic']:
+                a = b = c = (a+b+c)/3
+                alpha = beta = gamma = np.pi/2
+            elif ltype in ['hexagonal', 'trigonal', 'Hexagonal', 'Trigonal']:
+                a = b = (a+b)/2
+                alpha = beta = np.pi/2
+                gamma = np.pi*2/3
+            elif ltype in ['tetragonal', 'Tetragonal']:
+                a = b = (a+b)/2
+                alpha = beta = gamma = np.pi/2
+            elif ltype in ['orthorhombic', 'Orthorhombic']:
+                alpha = beta = gamma = np.pi/2
+            elif ltype in ['monoclinic', 'Monoclinic']:
+                alpha = gamma = np.pi/2
+            
+            # reset matrix according to the symmetry
+            m = para2matrix([a, b, c, alpha, beta, gamma])
+        
         # Initialize a Lattice instance
+        volume = np.linalg.det(m)
         l = Lattice(ltype, volume, PBC=PBC, **kwargs)
         l.a, l.b, l.c = a, b, c
         l.alpha, l.beta, l.gamma = alpha, beta, gamma
@@ -810,7 +889,7 @@ def generate_lattice_2D(
             v = random_vector()
             thickness1 = np.cbrt(volume) * (v[0] / (v[0] * v[1] * v[2]))
         else:
-            thickness1 = thickness
+            thickness1 = max([3.0, thickness])
         abc[NPA - 1] = thickness1
         alpha, beta, gamma = np.pi / 2, np.pi / 2, np.pi / 2
         # Triclinic
@@ -1292,7 +1371,8 @@ def matrix2para(matrix, radians=True):
     return cell_para
 
 
-def para2matrix(cell_para, radians=True, format="lower"):
+#def para2matrix(cell_para, radians=True, format="lower"):
+def para2matrix(cell_para, radians=True, format="upper"):
     """
     Given a set of lattic parameters, generates a matrix representing the
     lattice vectors
