@@ -48,11 +48,12 @@ class pyxtal_molecule:
    
     """
 
-    def __init__(self, mol, tm=Tol_matrix(prototype="molecular")):
+    def __init__(self, mol=None, symmetrize=True, tm=Tol_matrix(prototype="molecular")):
         mo = None
         if type(mol) == str:
             # Parse molecules: either file or molecule name
             tmp = mol.split(".")
+            self.name = tmp[0]
             if len(tmp) > 1:
                 # Load the molecule from the given file
                 if tmp[-1] in ["xyz", "gjf", "g03", "json"]:
@@ -67,6 +68,7 @@ class pyxtal_molecule:
                 # print('\nLoad the molecule {:s} from collections'.format(mol))
                 mo = molecule_collection[mol]
         elif hasattr(mol, "sites"):  # pymatgen molecule
+            self.name = str(mol.formula)
             mo = mol
 
         if mo is None:
@@ -76,8 +78,9 @@ class pyxtal_molecule:
         self.props = mo.site_properties
 
         if len(mo) > 1:
-            pga = PointGroupAnalyzer(mo)
-            mo = pga.symmetrize_molecule()["sym_mol"] 
+            if symmetrize:
+                pga = PointGroupAnalyzer(mo)
+                mo = pga.symmetrize_molecule()["sym_mol"] 
             mo = self.add_site_props(mo)
 
         self.mol = mo
@@ -88,6 +91,39 @@ class pyxtal_molecule:
         self.get_symbols()
         self.get_tols_matrix()
 
+    def __str__(self):
+        return '[' + self.name + ']'
+
+    def save_dict(self):
+        return self.mol.as_dict()
+
+    def copy(self):
+        """
+        simply copy the structure
+        """
+        return deepcopy(self)
+
+    def reset_positions(self, coors):
+        """
+        reset the coordinates
+        """
+        from pymatgen.core.sites import Site
+        if len(coors) != len(self.mol._sites):
+            raise ValueError("number of atoms is inconsistent!")
+        else:
+            for i, coor in enumerate(coors):
+                _site = self.mol._sites[i]
+                new_site = Site(_site.species, coor, properties=_site.properties)
+                self.mol._sites[i] = new_site
+
+    @classmethod
+    def load_dict(cls, dicts):
+        """
+        load the molecule from a dictionary
+        """
+        mol = Molecule.from_dict(dicts)
+        return cls(mol)
+
     def swap_axis(self, ax):
         """
         swap the molecular axis
@@ -98,7 +134,6 @@ class pyxtal_molecule:
 
         return pyxtal_molecule(mo, self.tm)
 
-
     def add_site_props(self, mo):
         if len(self.props) > 0:
             for key in self.props.keys():
@@ -240,9 +275,8 @@ class Orientation:
             if degrees is equal to 1
     """
 
-    def __init__(self, matrix, degrees=2, axis=None):
+    def __init__(self, matrix=None, degrees=2, axis=None):
         self.matrix = np.array(matrix)
-        self.r = Rotation.from_matrix(matrix)  # scipy transform.Rotation class
         self.degrees = degrees  # The number of degrees of freedom.
         if degrees == 1:
             if axis is None:
@@ -250,6 +284,8 @@ class Orientation:
             else:
                 axis /= np.linalg.norm(axis)
         self.axis = axis
+
+        self.r = Rotation.from_matrix(self.matrix)  # scipy transform.Rotation class
         self.angle = None
 
     def __str__(self):
@@ -264,13 +300,30 @@ class Orientation:
             s += "{:6.2f} {:6.2f} {:6.3f}\n".format(*self.axis)
         return s
 
+    def reset_matrix(self, matrix):
+        self.matrix = matrix
+        self.r = Rotation.from_matrix(matrix)
+
     def __repr__(self):
         return str(self)
 
-
     def copy(self):
         return deepcopy(self)
 
+    def save_dict(self):
+        dict0 = {"matrix": self.matrix,
+                 "degrees": self.degrees,
+                 "axis": self.axis
+                }
+        return dict0
+    
+    @classmethod
+    def load_dict(cls, dicts):
+        matrix = dicts['matrix']
+        degrees = dicts['degrees']
+        axis = dicts['axis']
+        return cls(matrix, degrees, axis)
+
     def change_orientation(self, angle="random", flip=False):
         """
         Allows for specification of an angle (possibly random) to
@@ -804,15 +857,15 @@ def orientation_in_wyckoff_position(
                     if pga.is_valid_op(old_op):
                         list_i.remove(j)
                         list_j.remove(j)
-    copy = deepcopy(orientations)
-    orientations = []
+    #copies = deepcopy(orientations)
+    orientations_new = []
     for i in list_i:
-        orientations.append(copy[i])
+        orientations_new.append(orientations[i])
 
     #Check each of the found orientations for consistency with the Wyckoff pos.
     #If consistent, put into an array of valid orientations
     allowed = []
-    for o in orientations:
+    for o in orientations_new:
         if randomize is True:
             op = o.get_op("random")
         elif randomize is False:
