@@ -364,7 +364,46 @@ def get_inverse(op):
     Returns:
         the inverse
     """
-    return SymmOp(np.linalg.inv(op.affine_matrix))
+    matrix = op.affine_matrix.copy()
+    # fill the matrix if it is ill conditioned
+    # experimental
+    if np.linalg.matrix_rank(matrix) < 4:
+        for row in range(3):
+            # fixed value
+            if np.sum(matrix[row,:3]**2) < 1e-3:
+                matrix[row, row] = 1
+                matrix[row, 3] = 0
+
+        if np.linalg.matrix_rank(matrix) == 3:
+            # [-3x/2, -x/2, 1/4]
+            # [0, x, 1/4]
+            for rows in [[0,1,2],[1,2,0],[0,2,1]]:
+                #m = (matrix[rows,:])[:,rows] 
+                #print(rows, m)
+                if np.linalg.matrix_rank(matrix[rows[:2],:3]) != 2:
+                    break
+            id0, id1, id2 = rows[0], rows[1], rows[2]
+            if matrix[id0, id1] == 0:
+                matrix[id0, id1], matrix[id0, id0] = matrix[id0, id0], matrix[id0, id1]
+                if np.linalg.matrix_rank(matrix) == 3:
+                    matrix[id0, id1], matrix[id0, id2] = matrix[id0, id2], matrix[id0, id1]
+            else:
+                matrix[id1, id0], matrix[id1, id1] = matrix[id1, id1], matrix[id1, id0]
+                if np.linalg.matrix_rank(matrix) == 3:
+                    matrix[id1, id0], matrix[id1, id2] = matrix[id1, id2], matrix[id1, id0]
+
+        elif np.linalg.matrix_rank(matrix) == 2:
+            # -3x/2, -x/2, -x+1/4 
+            if np.sum(matrix[:, 0]**2) > 1e-3:
+                matrix[1,0], matrix[1,1] = matrix[1,1], matrix[1,0]
+                matrix[2,0], matrix[2,2] = matrix[2,2], matrix[2,0]
+            elif np.sum(matrix[:, 1]**2) > 1e-3:
+                matrix[0,1], matrix[0,0] = matrix[0,0], matrix[0,1]
+                matrix[2,1], matrix[2,2] = matrix[2,2], matrix[2,1]
+            else:
+                matrix[0,2], matrix[0,0] = matrix[0,0], matrix[0,2]
+                matrix[1,2], matrix[1,1] = matrix[1,1], matrix[1,2]
+    return SymmOp(np.linalg.inv(matrix))
 
 
 def get_inverse_ops(ops):
