@@ -19,8 +19,9 @@ from pyxtal.operations import (
     project_point, 
     filtered_coords, 
     create_matrix,
+    SymmOp,
 )
-from pyxtal.symmetry import Group, jk_from_i
+from pyxtal.symmetry import Group, jk_from_i, Wyckoff_position
 from pyxtal.symmetry import ss_string_from_ops as site_symm
 from pyxtal.database.element import Element
 from pyxtal.constants import rad, deg
@@ -43,31 +44,28 @@ class mol_site:
         diag: whether or not use the `n` representation
     """
 
-    def __init__(self, mol, position, orientation, wp, lattice, diag=False):
+    def __init__(self, mol, position, orientation, wp, lattice=None, diag=False):
         # describe the molecule
         self.molecule = mol
+        self.diag = diag
+        self.wp = wp
+        self.position = position # fractional coordinate of molecular center
+        self.orientation = orientation #pyxtal.molecule.orientation object 
+        if isinstance(lattice, Lattice):
+            self.lattice = lattice
+        else:
+            self.lattice = Lattice.from_matrix(lattice)
+        self.PBC = self.wp.PBC
         self.mol = mol.mol # A Pymatgen molecule object
         self.site_props = mol.props
         self.symbols = mol.symbols #[site.specie.value for site in self.mol.sites]
         self.numbers = self.mol.atomic_numbers
         self.tols_matrix = mol.tols_matrix
         self.radius = mol.radius
-        self.diag = diag
-
-        self.position = position # fractional coordinate of molecular center
-        self.orientation = orientation #pyxtal.molecule.orientation object 
-        self.PBC = wp.PBC #The periodic axes
 
         if self.diag:
-            wp.diagonalize_symops()
+            self.wp.diagonalize_symops()
             self.position = project_point(self.position, wp[0])
-        self.wp = wp
-
-        if isinstance(lattice, Lattice):
-            self.lattice = lattice
-        else:
-            self.lattice = Lattice.from_matrix(lattice)
-
 
     def __str__(self):
 
@@ -77,18 +75,56 @@ class mol_site:
             )
         self.angles = self.orientation.r.as_euler('zxy', degrees=True)
         formula = self.mol.formula.replace(" ","")
-        s = "{:} @ [{:6.4f} {:6.4f} {:6.4f}]  ".format(formula, *self.position)
+        s = "{:} @ [{:7.4f} {:7.4f} {:7.4f}]  ".format(formula, *self.position)
         s += "WP: {:2d}{:s}, ".format(self.wp.multiplicity, self.wp.letter)
         s += "Site symmetry {:} ==> Euler: ".format(self.site_symm)
         s += "{:6.3f} {:6.3f} {:6.3f}".format(*self.angles)
         return s
 
+    def _get_dof(self):
+        """
+        get the number of dof for the given structures:
+        """
+        freedom = np.trace(self.wp.ops[0].rotation_matrix) > 0
+        self.dof = len(freedom[freedom==True])
+
+
+    def save_dict(self):
+        dict0 = {"position": self.position,
+                 "number": self.wp.number,
+                 "dim": self.wp.dim,
+                 "index": self.wp.index,
+                 "diag": self.diag,
+                 "molecule": self.molecule.save_dict(),
+                 "orientation": self.orientation.save_dict(),
+                 "lattice": self.lattice.matrix,
+                }
+        return dict0
+
+    @classmethod
+    def load_dict(cls, dicts):
+        """
+        load the sites from a dictionary
+        """
+        from pyxtal.molecule import pyxtal_molecule, Orientation
+
+        g = dicts["number"]
+        index = dicts["index"]
+        dim = dicts["dim"]
+        mol = pyxtal_molecule.load_dict(dicts["molecule"])
+        position = dicts["position"]
+        orientation = Orientation.load_dict(dicts['orientation'])
+        wp = Wyckoff_position.from_group_and_index(g, index, dim)
+        diag = dicts["diag"]
+        lattice = Lattice.from_matrix(dicts["lattice"])
+        return cls(mol, position, orientation, wp, lattice, diag)
+
 
     def show(self, id=None, **kwargs):
         from pyxtal.viz import display_molecular_site
         return display_molecular_site(self, id, **kwargs)
 
-    def _get_coords_and_species(self, absolute=False, add_PBC=False, first=False):
+    def _get_coords_and_species(self, absolute=False, add_PBC=False, first=False, unitcell=False):
         """
         Used to generate coords and species for get_coords_and_species
 
@@ -98,6 +134,7 @@ class mol_site:
             add_PBC: whether or not to add coordinates in neighboring unit cells, 
                 used for distance checking
             first: whether or not to extract the information from only the first site
+            unitcell: whether or not to move the molecular center to the unit cell
 
         Returns:
             atomic coords: a numpy array of fractional coordinates for the atoms in the site
@@ -110,16 +147,17 @@ class mol_site:
         for point_index, op2 in enumerate(self.wp.ops):
             # Obtain the center in absolute coords
             center_relative = op2.operate(self.position)
+            if unitcell:
+                center_relative -= np.floor(center_relative)
             center_absolute = np.dot(center_relative, self.lattice.matrix)
 
             # Rotate the molecule (Euclidean metric)
-            #op2_m = self.wp.generators_m[point_index]
             op2_m = self.wp.generators_m[point_index]
-            rot = op2_m.affine_matrix[0:3][:, 0:3].T
+            rot = op2_m.affine_matrix[:3, :3].T
             if self.diag and self.wp.index > 0:
                 tau = op2.translation_vector
             else:
-                tau = op2_m.affine_matrix[0:3][:, 3]
+                tau = op2_m.affine_matrix[:3, 3]
             tmp = np.dot(coord0, rot) + tau
             # Add absolute center to molecule
             tmp += center_absolute
@@ -150,7 +188,7 @@ class mol_site:
 
         return wp_atomic_coords, wp_atomic_sites
 
-    def get_coords_and_species(self, absolute=False, add_PBC=False):
+    def get_coords_and_species(self, absolute=False, add_PBC=False, unitcell=False):
         """
         Lazily generates and returns the atomic coordinate and species for the
         Wyckoff position. Plugs the molecule into the provided orientation
@@ -161,12 +199,13 @@ class mol_site:
                 coordinates. If false, return relative coordinates instead
             add_PBC: whether or not to add coordinates in neighboring unit cells, used for
                 distance checking
+            unitcell: whether or not to move the molecular center to the unit cell
 
         Returns:
             coords: a np array of 3-vectors.
             species: a list of atomic symbols, e.g. ['H', 'H', 'O', 'H', 'H', 'O']
         """
-        return self._get_coords_and_species(absolute, add_PBC)
+        return self._get_coords_and_species(absolute, add_PBC, unitcell=unitcell)
 
     def get_centers(self, absolute=False):
         """
@@ -222,14 +261,23 @@ class mol_site:
         matrix = self.get_principle_axes(coord0, True)
         return R.from_matrix(matrix).as_euler('zxy', degrees=True)
 
-    def perturbate(self, magnitude=1.0):
+    def perturbate(self, lattice, trans=0.1, rot=5):
         """
         Random perturbation of the molecular site
+        
+        Args:
+            lattice: lattice vectors
+            trans: magnitude of tranlation vectors (default: 0.1 A)
+            rot: magnitude of rotation degree (default: 5.0)
         """
-        disp = np.random.random([1,3])-0.5
-        self.translate(magnitude*disp, True)
-        self.orientation.change_orientation()
-
+        dis = (np.random.sample(3) - 0.5).dot(lattice)
+        dis /= np.linalg.norm(dis)
+        dis *= trans
+        self.translate(dis, True)
+        if rot == 'random':
+            self.orientation.change_orientation()
+        else:
+            self.orientation.change_orientation(angle=rot/180*np.pi)
     
     def translate(self, disp=np.array([0.0,0.0,0.0]), absolute=False):
         """
@@ -241,7 +289,6 @@ class mol_site:
         position = self.position + disp
         self.position = project_point(position, self.wp[0])
 
-
     def rotate(self, axis=0, angle=180):
         """
         To rotate the molecule 
@@ -289,7 +336,6 @@ class mol_site:
             raise ValueError("id is greater than the number of molecules")
         
 
-
     def update(self, coords, lattice=None, absolute=False, update_mol=True):
         """
         After the geometry relaxation, the returned atomic coordinates
@@ -344,7 +390,6 @@ class mol_site:
             #print(self.mol.to(fmt='xyz'))
             raise ValueError("molecular connectivity changes! Exit")
         #todo check if connectivty changed
-        
    
     def _find_gen_wyckoff_in_subgroup(self, groups=None):
         """
@@ -526,60 +571,60 @@ class mol_site:
 
 
 
-def check_mol_sites(ms1, ms2, factor=1.0, tm=Tol_matrix(prototype="molecular")):
-    """
-    Checks whether or not the molecules of two mol sites overlap. Uses
-    ellipsoid overlapping approximation to check. Takes PBC and lattice
-    into consideration.
-
-    Args:
-        ms1: a mol_site object
-        ms2: another mol_site object
-        factor: the distance factor to pass to check_distances. (only for
-            inter-atomic distance checking)
-        tm: a Tol_matrix object (or prototype string) for distance checking
-
-    Returns:
-        False if the Wyckoff positions overlap. True otherwise
-    """
-    # Get coordinates for both mol_sites
-    c1, _ = ms1.get_coords_and_species()
-    c2, _ = ms2.get_coords_and_species()
-
-    # Calculate which distance matrix is smaller/faster
-    m_length1 = len(ms1.numbers)
-    m_length2 = len(ms2.numbers)
-    wp_length1 = len(c1)
-    wp_length2 = len(c2)
-    size1 = m_length1 * wp_length2
-    size2 = m_length2 * wp_length1
-
-    # Case 1
-    if size1 <= size2:
-        coords_mol = c1[:m_length1]
-        # Calculate tol matrix for species pairs
-        tols = np.zeros((m_length1, m_length2))
-        for i1, number1 in enumerate(ms1.numbers):
-            for i2, number2 in enumerate(ms2.numbers):
-                tols[i1][i2] = tm.get_tol(number1, number2)
-        tols = np.repeat(tols, ms2.wp.multiplicity, axis=1)
-        d = distance_matrix(coords_mol, c2, ms1.lattice.matrix, PBC=ms1.PBC)
-
-    # Case 2
-    elif size1 > size2:
-        coords_mol = c2[:m_length2]
-        # Calculate tol matrix for species pairs
-        tols = np.zeros((m_length2, m_length1))
-        for i1, number1 in enumerate(ms2.numbers):
-            for i2, number2 in enumerate(ms1.numbers):
-                tols[i1][i2] = tm.get_tol(number1, number2)
-        tols = np.repeat(tols, ms1.wp.multiplicity, axis=1)
-        d = distance_matrix(coords_mol, c1, ms1.lattice.matrix, PBC=ms1.PBC)
-
-    # Check if distances are smaller than tolerances
-    if (d < tols).any():
-        return False
-    return True
+    def check_with_ms2(self, ms2, factor=1.0, tm=Tol_matrix(prototype="molecular")):
+        """
+        Checks whether or not the molecules of two mol sites overlap. Uses
+        ellipsoid overlapping approximation to check. Takes PBC and lattice
+        into consideration.
+    
+        Args:
+            ms1: a mol_site object
+            ms2: another mol_site object
+            factor: the distance factor to pass to check_distances. (only for
+                inter-atomic distance checking)
+            tm: a Tol_matrix object (or prototype string) for distance checking
+    
+        Returns:
+            False if the Wyckoff positions overlap. True otherwise
+        """
+        # Get coordinates for both mol_sites
+        c1, _ = self.get_coords_and_species()
+        c2, _ = ms2.get_coords_and_species()
+    
+        # Calculate which distance matrix is smaller/faster
+        m_length1 = len(self.numbers)
+        m_length2 = len(ms2.numbers)
+        wp_length1 = len(c1)
+        wp_length2 = len(c2)
+        size1 = m_length1 * wp_length2
+        size2 = m_length2 * wp_length1
+    
+        # Case 1
+        if size1 <= size2:
+            coords_mol = c1[:m_length1]
+            # Calculate tol matrix for species pairs
+            tols = np.zeros((m_length1, m_length2))
+            for i1, number1 in enumerate(self.numbers):
+                for i2, number2 in enumerate(ms2.numbers):
+                    tols[i1][i2] = tm.get_tol(number1, number2)
+            tols = np.repeat(tols, ms2.wp.multiplicity, axis=1)
+            d = distance_matrix(coords_mol, c2, self.lattice.matrix, PBC=self.PBC)
+    
+        # Case 2
+        elif size1 > size2:
+            coords_mol = c2[:m_length2]
+            # Calculate tol matrix for species pairs
+            tols = np.zeros((m_length2, m_length1))
+            for i1, number1 in enumerate(ms2.numbers):
+                for i2, number2 in enumerate(self.numbers):
+                    tols[i1][i2] = tm.get_tol(number1, number2)
+            tols = np.repeat(tols, self.wp.multiplicity, axis=1)
+            d = distance_matrix(coords_mol, c1, self.lattice.matrix, PBC=self.PBC)
+    
+        # Check if distances are smaller than tolerances
+        if (d < tols).any():
+            return False
+        return True
 
 
 class atom_site:
@@ -590,85 +635,199 @@ class atom_site:
         wp: a `Wyckoff_position <pyxtal.symmetry.Wyckoff_position.html> object 
         coordinate: a fractional 3-vector for the generating atom's coordinate
         specie: an Element, element name or symbol, or atomic number of the atom
+        search: to search for the optimum position for special wyckoff site
     """
 
-    def __init__(self, wp, coordinate, specie=1):
+    def __init__(self, wp=None, coordinate=None, specie=1, diag=False, search=False):
         self.position = np.array(coordinate)
         self.specie = Element(specie).short_name
-        self.multiplicity = wp.multiplicity
+        self.diag = diag
         self.wp = wp
-        self.PBC = wp.PBC
-        self.update(coordinate)
+        if self.diag:
+            self.wp.diagonalize_symops()
+            #self.position = project_point(self.position, wp[0])
+
+        self._get_dof()
+        self.PBC = self.wp.PBC
+        self.multiplicity = self.wp.multiplicity
+        if search:
+            self.search_position()
+        self.update()
 
     def __str__(self):
+        #number, dim = self.wp.number, self.wp.dim
+        #self.site_symm = site_symm(self.wp.symmetry_m[0], number, dim=dim)
+
         if not hasattr(self, "site_symm"):
             self.site_symm = site_symm(
                 self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
             )
-
-        s = "{:>2s} @ [{:6.4f} {:6.4f} {:6.4f}], ".format(self.specie, *self.position)
+        s = "{:>2s} @ [{:7.4f} {:7.4f} {:7.4f}], ".format(self.specie, *self.position)
         s += "WP: {:2d}{:s}, ".format(self.wp.multiplicity, self.wp.letter)
         s += "Site symmetry: {:s}".format(self.site_symm)
         return s
 
-    def update(self, pos):
-        """
-        Used to generate coords from self.position
-        """
-        self.position = pos
-        self.coords = apply_ops(pos, self.wp) 
-
     def __repr__(self):
         return str(self)
 
+    def save_dict(self):
+        dict0 = {"position": self.position,
+                 "specie": self.specie,
+                 "number": self.wp.number,
+                 "dim": self.wp.dim,
+                 "index": self.wp.index,
+                 "PBC": self.wp.PBC,
+                }
+        return dict0
 
-def check_atom_sites(ws1, ws2, lattice, tm, same_group=True):
-    """
-    Given two Wyckoff sites, checks the inter-atomic distances between them.
+    def _get_dof(self):
+        """
+        get the number of dof for the given structures:
+        """
+        freedom = np.trace(self.wp.ops[0].rotation_matrix) > 0
+        self.dof = len(freedom[freedom==True])
 
-    Args:
-        ws1: a Wyckoff_site object
-        ws2: a different Wyckoff_site object (will always return False if
+
+    @classmethod
+    def load_dict(cls, dicts):
+        """
+        load the sites from a dictionary
+        """
+        g = dicts["number"]
+        index = dicts["index"]
+        dim = dicts["dim"]
+        PBC = dicts["PBC"]
+        position = dicts["position"]
+        specie = dicts["specie"]
+        wp = Wyckoff_position.from_group_and_index(g, index, dim, PBC)
+        return cls(wp, position, specie)
+
+    def perturbate(self, lattice, magnitude=0.1):
+        """
+        Random perturbation of the site
+        
+        Args:
+            lattice: lattice vectors
+            magnitude: the magnitude of displacement (default: 0.1 A)
+        """
+        dis = (np.random.sample(3) - 0.5).dot(lattice)
+        dis /= np.linalg.norm(dis)
+        dis *= magnitude
+        pos = self.position + dis.dot(np.linalg.inv(lattice))
+        self.update(pos)
+ 
+    def search_position(self):
+        """
+        Sometimes, the initial posiition is not the proper generator
+        Needs to find the proper generator
+        """
+        if self.wp.index > 0:
+            pos = self.position
+            coords = apply_ops(pos, Group(self.wp.number, self.wp.dim)[0])
+            for coord in coords:
+                ans = apply_ops(coord, [self.wp.ops[0]])[0]
+                diff = coord - ans
+                diff -= np.floor(diff)
+                if np.sum(diff**2)<1e-4:
+                    self.position = coord - np.floor(coord)
+                    break
+    def swap_axis(self, swap_id, shift=np.zeros(3)):
+        """
+        sometimes space groups like Pmm2 allows one to swap the a,b axes
+        to get an alternative representation
+        """
+        self.position += shift
+        self.position = self.position[swap_id]
+        self.position -= np.floor(self.position)
+        self.wp, _ = self.wp.swap_axis(swap_id)
+        self.site_symm = site_symm(
+            self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
+        )
+        self.update()
+
+    def shift_by_swap(self, swap_id):
+        """
+        check if a shift is needed during swap
+        May occur for special WP in the I/A/B/C/F cases
+        e.g., in space group 71 (Immm), the permutation
+        4j(1/2,0,0.2) -> (0.2,0,1/2) -> 4f(0.7,1/2,0)
+        it requires a shift of (0.5,0.5,0.5)
+        """
+        wp, shift = self.wp.swap_axis(swap_id)
+        return shift
+
+    def equivalent_set(self, tran, indices):
+        """
+        Transform the wp to another equivalent set.
+        Needs to update both wp and positions
+
+        Args:
+            tran: affine matrix
+            indices: the list of transformed wps 
+        """
+        self.position = SymmOp(tran).operate(self.position)
+        self.position -= np.floor(self.position)
+        self.wp = self.wp.equivalent_set(indices[self.wp.index]) #update the wp index
+        self.site_symm = site_symm(
+            self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
+        )
+        self.update()
+
+
+    def update(self, pos=None):
+        """
+        Used to generate coords from self.position
+        """
+        if pos is None:
+            pos = self.position
+        self.coords = apply_ops(pos, self.wp) 
+        self.position = self.coords[0]
+
+    def check_with_ws2(self, ws2, lattice, tm, same_group=True):
+        """
+        Given two Wyckoff sites, checks the inter-atomic distances between them.
+
+        Args:
+            ws2: a different Wyckoff_site object (will always return False if
             two identical WS's are provided)
-        lattice: a 3x3 cell matrix
-        same_group: whether or not the two WS's are in the same structure.
+            lattice: a 3x3 cell matrix
+            same_group: whether or not the two WS's are in the same structure.
             Default value True reduces the calculation cost
-
-    Returns:
-        True if all distances are greater than the allowed tolerances.
-        False if any distance is smaller than the allowed tolerance
-    """
-    # Ensure the PBC values are valid
-    if ws1.PBC != ws2.PBC:
-        printx("Error: PBC values do not match between Wyckoff sites")
-        return
-    # Get tolerance
-    tol = tm.get_tol(ws1.specie, ws2.specie)
-    # Symmetry shortcut method: check only some atoms
-    if same_group is True:
-        # We can either check one atom in WS1 against all WS2, or vice-versa
-        # Check which option is faster
-        if ws1.multiplicity > ws2.multiplicity:
-            coords1 = [ws1.coords[0]]
-            coords2 = ws2.coords
+        Returns:
+            True if all distances are greater than the allowed tolerances.
+            False if any distance is smaller than the allowed tolerance
+        """
+        # Ensure the PBC values are valid
+        if self.PBC != ws2.PBC:
+            printx("Error: PBC values do not match between Wyckoff sites")
+            return
+        # Get tolerance
+        tol = tm.get_tol(self.specie, ws2.specie)
+        # Symmetry shortcut method: check only some atoms
+        if same_group is True:
+            # We can either check one atom in WS1 against all WS2, or vice-versa
+            # Check which option is faster
+            if self.multiplicity > ws2.multiplicity:
+                coords1 = [self.coords[0]]
+                coords2 = ws2.coords
+            else:
+                coords1 = [ws2.coords[0]]
+                coords2 = self.coords
+            # Calculate distances
+            dm = distance_matrix(coords1, coords2, lattice, PBC=self.PBC)
+            # Check if any distances are less than the tolerance
+            if (dm < tol).any():
+                return False
+            else:
+                return True
+        # No symmetry method: check all atomic pairs
         else:
-            coords1 = [ws2.coords[0]]
-            coords2 = ws1.coords
-        # Calculate distances
-        dm = distance_matrix(coords1, coords2, lattice, PBC=ws1.PBC)
-        # Check if any distances are less than the tolerance
-        if (dm < tol).any():
-            return False
-        else:
-            return True
-    # No symmetry method: check all atomic pairs
-    else:
-        dm = distance_matrix(ws1.coords, ws2.coords, lattice, PBC=ws1.PBC)
-        # Check if any distances are less than the tolerance
-        if (dm < tol).any():
-            return False
-        else:
-            return True
+            dm = distance_matrix(ws1.coords, ws2.coords, lattice, PBC=ws1.PBC)
+            # Check if any distances are less than the tolerance
+            if (dm < tol).any():
+                return False
+            else:
+                return True
 
 def WP_merge(pt, lattice, wp, tol, orientations=None):
     """
@@ -687,7 +846,7 @@ def WP_merge(pt, lattice, wp, tol, orientations=None):
 
     Returns:
         pt: 3-vector after merge
-        wp: a pyxtal.symmetry.Wyckoff_position object, If no matching WP, returns False. 
+        wp: a `pyxtal.symmetry.Wyckoff_position` object, If no matching WP, returns False. 
         valid_ori: the valid orientations after merge
 
     """
