@@ -1,7 +1,7 @@
 #!/usr/bin/env  python
 # encoding: utf-8
 
-# Test script for pyXtal version 0.1dev. Tests core functions for all modules.
+# Test script for pyXtal v-0.1.4. Tests core functions for all modules.
 
 
 import sys
@@ -20,17 +20,7 @@ from pyxtal.symmetry import (
     get_rod,
     get_point,
 )
-from pyxtal.crystal import (
-    random_crystal,
-    random_crystal_2D,
-    random_crystal_1D,
-    random_cluster,
-    cellsize,
-)
-from pyxtal.molecular_crystal import (
-    molecular_crystal,
-    molecular_crystal_2D,
-)
+from pyxtal import pyxtal
 from pyxtal.operations import distance, filtered_coords
 
 
@@ -180,7 +170,7 @@ def check_struct_group(crystal, group, dim=3, tol=1e-2):
 
         """Given a pymatgen structure, group number, and dimension, return
         whether or not the structure matches the group number."""
-        if isinstance(crystal, (random_crystal, molecular_crystal)):
+        if isinstance(crystal, pyxtal):
             pmg_struc = crystal.to_pymatgen()
             if dim > 0:
                 lattice = pmg_struc.lattice.matrix
@@ -257,18 +247,20 @@ def test_atomic():
     global outstructs
     global outstrings
     fprint("=== Testing generation of atomic 3D crystals. This may take some time. ===")
-    my_crystal1 = random_crystal(99, ['Ba','Ti','O'], [1,1,3], 1.0, sites=[["1b"], ["1b"], ["2c", "1b"]])
+    my_crystal1 = pyxtal()
+    my_crystal1.from_random(3, 99, ['Ba','Ti','O'], [1,1,3], 1.0, sites=[["1b"], ["1b"], ["2c", "1b"]])
     print(my_crystal1)
-    my_crystal1.to_file("1.cif")
+    #my_crystal1.to_file("1.cif")
     
-    my_crystal2 = random_crystal(225, ['C'], [3], 1.0, sites=[["4a", "8c"]])
-    my_crystal2 = random_crystal(225, ['C'], [3], 1.0)
+    my_crystal2 = pyxtal()
+    my_crystal2.from_random(3, 225, ['C'], [12], 1.0, sites=[["4a", "8c"]])
     print(my_crystal2)
-    my_crystal2.to_file("2.cif")
+    #my_crystal2.to_file("2.cif")
     
-    my_crystal3 = random_crystal(225, ['C','Si'], [3, 1], 1.0, sites=[["4a", "8c"], None])
+    my_crystal3 = pyxtal()
+    my_crystal3.from_random(3, 225, ['C','Si'], [12, 4], 1.0, sites=[["4a", "8c"], None])
     print(my_crystal3)
-    my_crystal3.to_file("3.cif")
+    #my_crystal3.to_file("3.cif")
 
 
     slow = []
@@ -281,11 +273,10 @@ def test_atomic():
     # )
     for sg in range(1, 231):
         if sg not in skip:
-            multiplicity = len(get_wyckoffs(sg)[0]) / cellsize(
-                sg
-            )  # multiplicity of the general position
+            multiplicity = len(get_wyckoffs(sg)[0]) 
             start = time()
-            rand_crystal = random_crystal(sg, ["C"], [multiplicity], 1.0)
+            rand_crystal = pyxtal()
+            rand_crystal.from_random(3, sg, ["C"], [multiplicity], 1.0)
             end = time()
             timespent = np.around((end - start), decimals=2)
             t = str(timespent)
@@ -376,11 +367,10 @@ def test_molecular():
     ]  # [24, 183, 202, 203, 209, 210, 216, 219, 225, 226, 227, 228, 229, 230] #slow
     for sg in range(1, 231):
         if sg not in skip:
-            multiplicity = len(get_wyckoffs(sg)[0]) / cellsize(
-                sg
-            )  # multiplicity of the general position
+            multiplicity = len(get_wyckoffs(sg)[0])
             start = time()
-            rand_crystal = molecular_crystal(sg, ["H2O"], [multiplicity], 2.5)
+            rand_crystal = pyxtal(molecular=True)
+            rand_crystal.from_random(3, sg, ["H2O"], [multiplicity], 2.5)
             end = time()
             timespent = np.around((end - start), decimals=2)
             t = str(timespent)
@@ -470,7 +460,8 @@ def test_atomic_2D():
             g = Group(sg, dim=2)
             multiplicity = len(g[0])  # multiplicity of the general position
             start = time()
-            rand_crystal = random_crystal_2D(sg, ["C"], [multiplicity], 4.0)
+            rand_crystal = pyxtal()
+            rand_crystal.from_random(2, sg, ["C"], [multiplicity], 4.0)
             end = time()
             timespent = np.around((end - start), decimals=2)
             t = str(timespent)
@@ -526,7 +517,8 @@ def test_molecular_2D():
             g = Group(sg, dim=2)
             multiplicity = len(g[0])  # multiplicity of the general position
             start = time()
-            rand_crystal = molecular_crystal_2D(sg, ["H2O"], [multiplicity], 4.0)
+            rand_crystal = pyxtal(molecular=True)
+            rand_crystal.from_random(2, sg, ["H2O"], [multiplicity], 4.0)
             end = time()
             timespent = np.around((end - start), decimals=2)
             t = str(timespent)
@@ -579,7 +571,8 @@ def test_atomic_1D():
         if num not in skip:
             multiplicity = len(get_rod(num)[0])  # multiplicity of the general position
             start = time()
-            rand_crystal = random_crystal_1D(num, ["H"], [multiplicity], 4.0)
+            rand_crystal = pyxtal()
+            rand_crystal.from_random(1, num, ["H"], [multiplicity], 4.0)
             end = time()
             timespent = np.around((end - start), decimals=2)
             t = str(timespent)
@@ -646,11 +639,6 @@ def test_molecular_1D():
     fprint(
         "=== Testing generation of molecular 1D crystals. This may take some time. ==="
     )
-    from time import time
-    from spglib import get_symmetry_dataset
-    from pyxtal.symmetry import get_rod
-    from pyxtal.molecular_crystal import molecular_crystal_1D
-    from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 
     slow = []
     failed = []
@@ -660,7 +648,8 @@ def test_molecular_1D():
         if num not in skip:
             multiplicity = len(get_rod(num)[0])  # multiplicity of the general position
             start = time()
-            rand_crystal = molecular_crystal_1D(num, ["H2O"], [multiplicity], 4.0)
+            rand_crystal = pyxtal(molecular=True)
+            rand_crystal.from_random(1, num, ["H2O"], [multiplicity], 4.0)
             end = time()
             timespent = np.around((end - start), decimals=2)
             t = str(timespent)
@@ -736,7 +725,8 @@ def test_cluster():
                 Group(sg, dim=0)[0]
             )  # multiplicity of the general position
             start = time()
-            rand_crystal = random_cluster(sg, ["C"], [multiplicity], 1.0)
+            rand_crystal = pyxtal()
+            rand_crystal.from_random(0, sg, ["C"], [multiplicity], 1.0)
             end = time()
             timespent = np.around((end - start), decimals=2)
             t = str(timespent)
@@ -835,7 +825,7 @@ def test_modules():
         fail(e)
         sys.exit(0)
 
-    fprint("Importing openbabel...")
+    fprint("Importing ase...")
     try:
         import ase
 
@@ -843,15 +833,6 @@ def test_modules():
     except:
         fprint("Error: could not import openbabel. Try reinstalling the package.")
 
-    fprint("Importing pyxtal...")
-    try:
-        import pyxtal
-
-        fprint("Success!")
-    except Exception as e:
-        fail(e)
-        sys.exit(0)
-
     fprint("=== Testing modules ===")
 
     # =====database.element=====
@@ -1241,61 +1222,9 @@ def test_modules():
 
     check()
 
-    # =====crystal=====
-    fprint("pyxtal.crystal")
-    reset()
-    try:
-        import pyxtal.crystal
-    except Exception as e:
-        fail(e)
-
-    fprint("  random_crystal")
-    try:
-        from pyxtal.crystal import random_crystal
-    except Exception as e:
-        fail(e)
-
-    if passed():
-        try:
-            c = random_crystal(1, ["H"], [1], 10.0)
-            if c.valid is True:
-                pass
-            else:
-                fail()
-        except Exception as e:
-            fail(e)
-
-    check()
-
-    fprint("  random_crystal_2D")
-    try:
-        from pyxtal.crystal import random_crystal_2D
-    except Exception as e:
-        fail(e)
-
-    if passed():
-        try:
-            c = random_crystal_2D(1, ["H"], [1], 10.0)
-            if c.valid is True:
-                pass
-            else:
-                fail()
-        except Exception as e:
-            fail(e)
-
-    check()
-
     # =====molecule=====
     fprint("pyxtal.molecule")
     reset()
-    try:
-        import pyxtal.molecule
-    except Exception as e:
-        fail(e)
-
-    check()
-
-    fprint("pyxtal_molecule")
     try:
         from pyxtal.molecule import pyxtal_molecule
     except Exception as e:
@@ -1342,50 +1271,6 @@ def test_modules():
 
     check()
 
-    # =====molecular_crystal=====
-    fprint("pyxtal.molecular_crystal")
-    reset()
-    try:
-        import pyxtal.crystal
-    except Exception as e:
-        fail(e)
-
-    fprint("  molecular_crystal")
-    try:
-        from pyxtal.molecular_crystal import molecular_crystal
-    except Exception as e:
-        fail(e)
-
-    if passed():
-        try:
-            c = molecular_crystal(1, ["H2O"], [1], 10.0)
-            if c.valid is True:
-                pass
-            else:
-                fail()
-        except Exception as e:
-            fail(e)
-
-    check()
-
-    fprint("  molecular_crystal_2D")
-    try:
-        from pyxtal.molecular_crystal import molecular_crystal_2D
-    except Exception as e:
-        fail(e)
-
-    if passed():
-        try:
-            c = molecular_crystal_2D(1, ["H2O"], [1], 10.0)
-            if c.valid is True:
-                pass
-            else:
-                fail()
-        except Exception as e:
-            fail(e)
-
-    check()
-
     end(condition=2)
 
 
